@@ -1064,7 +1064,7 @@ def generate_ds_params(inst_num, role=ReplicaRole.STANDALONE):
     return instance_data
 
 
-def get_ds_version():
+def get_ds_version(paths=None):
     """
     Return version of ns-slapd installed on this system. This is determined by the defaults.inf
     file, so is correct for the built and installed ns-slapd binary. This function works without
@@ -1072,16 +1072,21 @@ def get_ds_version():
 
     returns a string of the form: 1.3.4.8
     """
-    p = Paths()
-    return p.version
+    if paths is None:
+        paths = Paths()
+    return paths.version
 
 
-def ds_is_related(relation, ds_ver, *ver):
+def ds_is_related(relation, *ver, instance=None):
     """
     Return a result of a comparison between the current version of ns-slapd and a provided version.
     """
     ops = {'older': operator.lt,
            'newer': operator.ge}
+    if instance is None:
+        ds_ver = get_ds_version()
+    else:
+        ds_ver = get_ds_version(instance.ds_paths)
     if len(ver) > 1:
         for cmp_ver in ver:
             if cmp_ver.startswith(ds_ver[:3]):
@@ -1090,20 +1095,18 @@ def ds_is_related(relation, ds_ver, *ver):
         return ops[relation](LegacyVersion(ds_ver), LegacyVersion(ver[0]))
 
 
-def ds_is_older(*ver):
+def ds_is_older(*ver, instance=None):
     """
     Return True if the current version of ns-slapd is older than a provided version
     """
-    ds_ver = get_ds_version()
-    return ds_is_related('older', ds_ver, *ver)
+    return ds_is_related('older', *ver, instance=instance)
 
 
-def ds_is_newer(*ver):
+def ds_is_newer(*ver, instance=None):
     """
     Return True if the current version of ns-slapd is newer than a provided version
     """
-    ds_ver = get_ds_version()
-    return ds_is_related('newer', ds_ver, *ver)
+    return ds_is_related('newer', *ver, instance=instance)
 
 
 def gentime_to_datetime(gentime):
@@ -1140,11 +1143,28 @@ def getDateTime():
 
 
 def socket_check_open(host, port):
-    with closing(socket.socket(socket.AF_INET6, socket.SOCK_STREAM)) as sock:
-        if sock.connect_ex((host, port)) == 0:
-            return True
-        else:
-            return False
+    """
+    Check if a socket can be opened.  Need to handle cases where IPv6 is completely
+    disabled.
+    """
+    try:
+        # Trying IPv6 first ...
+        family = socket.AF_INET6
+        with closing(socket.socket(family, socket.SOCK_STREAM)) as sock:
+            if sock.connect_ex((host, port)) == 0:
+                return True
+            else:
+                return False
+    except OSError:
+        # No IPv6, adjust hostname, and try IPv4 ...
+        family = socket.AF_INET
+        if host == "::1":
+            host = "127.0.0.1"
+        with closing(socket.socket(family, socket.SOCK_STREAM)) as sock:
+            if sock.connect_ex((host, port)) == 0:
+                return True
+            else:
+                return False
 
 
 def ensure_bytes(val):
@@ -1377,3 +1397,14 @@ def copy_with_permissions(source, target):
     st = os.stat(source)
     os.chown(target, st[stat.ST_UID], st[stat.ST_GID])
     os.chmod(target, st[stat.ST_MODE])
+
+
+def cmp(self, x, y):
+    """
+    Replacement for built-in function cmp that was removed in Python 3
+
+    Compare the two objects x and y and return an integer according to
+    the outcome. The return value is negative if x < y, zero if x == y
+    and strictly positive if x > y.
+    """
+    return (x > y) - (x < y)
